package mi.hdm.shoppingList;

import mi.hdm.exceptions.InvalidIngredientException;
import mi.hdm.recipes.Ingredient;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * Shopping list that the user can add ingredients to. The elements on the list can be marked as done or undone.
 */
public class ShoppingList {
    private static final Logger log = LogManager.getLogger(ShoppingList.class);
    private final Map<Ingredient, Boolean> list;

    private static final ShoppingList shoppingList = new ShoppingList();

    private ShoppingList() {
        list = new HashMap<>();
    }

    public static ShoppingList getInstance() {
        return shoppingList;
    }

    public void clear() {
        list.clear();
        log.info("ShoppingList cleared successfully.");
    }

    public void addToShoppingList(Ingredient ingredient) {
        if (ingredient == null) {
            throw new InvalidIngredientException("Can not add ingredient that is null");
        }
        list.put(ingredient, false);
        log.info("Added {} to shopping list.", ingredient.getName());
    }

    public void flipStatus(Ingredient in) {
        boolean newStatus = !list.get(in);
        list.put(in, newStatus);
        log.info("Ingredient {} marked as {}.", in.getName(), newStatus ? "done" : "not done");
    }

    public void flipStatus(String name) {
        Ingredient in = getIngredientByName(name).orElseThrow(() -> new InvalidIngredientException("No ingredient with name " + name));
        ;
        flipStatus(in);
    }

    /**
     * This method removes all ingredients from the shopping list that are marked as done (map.get(ingredient) == true)
     */
    public void removeAllBought() {
        int removed = 0;
        for (Ingredient i : list.keySet()) {
            if (list.get(i)) list.remove(i);
            removed++;
        }
        log.info("Removed {} ingredients from shopping list", removed);
    }

    public Map<Ingredient, Boolean> getList() {
        return list;
    }

    private Optional<Ingredient> getIngredientByName(String name) {
        for (final Ingredient i : list.keySet()) {
            if (name.equals(i.getName())) {
                return Optional.of(i);
            }
        }
        return Optional.empty();
    }
}
